@@ -60,6 +60,73 @@ def _set_combobox_default_selection(combobox, choices, selected):
         combobox.current(index)
 
 
+class DataSetCollectionFrame(ttk.Frame):
+    """Dataset collection frame."""
+
+    def __init__(self, master, root):
+        super().__init__(master)
+        logger.info("Initialising {}".format(self))
+
+        self._sort_key = None
+        self._reverse_sort_order = False
+
+        # Make sure that the GUI expands/shrinks when the window is resized.
+        self.columnconfigure(0, weight=1)
+        self.rowconfigure(1, weight=1)
+
+        self.root = root
+
+        self.search_bar_frame = SearchBarFrame(self, root)
+        self.dataset_list_frame = DataSetListFrame(self, root)
+
+        # Layout the frame.
+        self.search_bar_frame.grid(row=0, column=0, sticky="nw")
+        self.dataset_list_frame.grid(row=1, column=0, sticky="news")
+
+    def refresh(self):
+        self.dataset_list_frame.refresh()
+
+
+class SearchBarFrame(ttk.Frame):
+    """Search bar frame."""
+
+    def __init__(self, master, root):
+        super().__init__(master)
+        logger.info("Initialising {}".format(self))
+
+        # Make sure that the GUI expands/shrinks when the window is resized.
+        self.columnconfigure(0, weight=1)
+
+        self.root = root
+
+        tag_filter_lbl = ttk.Label(self, text="Filter by tag")
+        self.selected_tag = tk.StringVar()
+        self.tag_options = ttk.Combobox(
+            self,
+            values=root.dataset_list_model.list_tags(),
+            textvariable=self.selected_tag,
+            state="readonly"
+        )
+        self.tag_options.bind('<<ComboboxSelected>>', self.set_tag_filter)
+        tag_clear_btn = ttk.Button(self, text="Clear", command=self.clear_tag_filter)  # NOQA
+
+        tag_filter_lbl.grid(row=0, column=0, sticky="w")
+        self.tag_options.grid(row=0, column=1, sticky="w")
+        tag_clear_btn.grid(row=0, column=2, sticky="w")
+
+    def set_tag_filter(self, event):
+        tag = self.selected_tag.get()
+        self.root.dataset_list_model.set_tag_filter(tag)
+        self.master.dataset_list_frame.refresh(reindex=False)
+        self.root.dataset_frame.refresh()
+
+    def clear_tag_filter(self):
+        self.tag_options.set("")
+        self.root.dataset_list_model.set_tag_filter(None)
+        self.master.dataset_list_frame.refresh(reindex=False)
+        self.root.dataset_frame.refresh()
+
+
 class DataSetListFrame(ttk.Frame):
     """List dataset frame."""
 
@@ -180,6 +247,9 @@ class DataSetFrame(ttk.Frame):
         self.dataset_title_frame = DataSetTitleFrame(self, root)
         self.dataset_title_frame.grid(row=0, column=0, sticky="ew")
 
+        self.dataset_tags_frame = DataSetTagsFrame(self, root)
+        self.dataset_tags_frame.grid(row=1, column=0, sticky="ew")
+
         self.dataset_metadata_frame = DataSetMetadataFrame(self, root)
         self.dataset_metadata_frame.grid(row=2, column=0, sticky="news")
 
@@ -195,6 +265,7 @@ class DataSetFrame(ttk.Frame):
     def refresh(self):
 
         self.dataset_title_frame.refresh()
+        self.dataset_tags_frame.refresh()
         self.dataset_metadata_frame.refresh()
         self.dataset_item_frame.refresh()
 
@@ -223,6 +294,31 @@ class DataSetTitleFrame(ttk.Frame):
         self.name.config(text=self.root.dataset_model.name)
 
 
+class DataSetTagsFrame(ttk.Frame):
+    """View dataset title."""
+
+    def __init__(self, master, root):
+        super().__init__(master)
+        logger.info("Initialising {}".format(self))
+
+        self.root = root
+
+        # Make sure that the GUI expands/shrinks when the window is resized.
+        self.columnconfigure(0, weight=1)
+        self.rowconfigure(0, weight=1)
+
+        self.tags = ttk.Label(self)  # NOQA
+        self.tags.grid(row=0, column=0, sticky="ew")
+
+        ttk.Separator(self, orient=tk.HORIZONTAL).grid(
+            row=1, column=0, sticky="ew", pady=(0, 4)
+        )
+
+    def refresh(self):
+        tags_text = "Tags: " + "; ".join(self.root.dataset_model.list_tags())
+        self.tags.config(text=tags_text)
+
+
 class DataSetItemsFrame(ttk.Frame):
     """View dataset items."""
 
@@ -267,14 +363,14 @@ class DataSetItemsFrame(ttk.Frame):
         logger.info("Refreshing {}".format(self))
         self.item_list.delete(*self.item_list.get_children())
 
-        for props in self.root.dataset_model.get_item_props_list():
-            values = [props["relpath"], props["size_str"]]
-            self.item_list.insert("", "end", values=values)
-
         # Skip if a dataset is not loaded.
         if self.root.dataset_model.name is None:
             return
 
+        for props in self.root.dataset_model.get_item_props_list():
+            values = [props["relpath"], props["size_str"]]
+            self.item_list.insert("", "end", values=values)
+
 
 class DataSetMetadataFrame(ttk.Frame):
     """View dataset metadata."""
@@ -285,7 +381,6 @@ class DataSetMetadataFrame(ttk.Frame):
 
         # Make sure that the GUI expands/shrinks when the window is resized.
         self.columnconfigure((0, 1), weight=1)
-#       self.rowconfigure(0, weight=1)
 
         self.root = root
         if self.root.base_uri_model.get_base_uri() is not None:
@@ -848,6 +943,108 @@ class EditMetadataFrame(ttk.Frame):
         self.metadata_form_frame.refresh()
 
 
+class TagListFrame(ttk.Frame):
+
+    def __init__(self, master):
+        super().__init__(master)
+        logger.info("Initialising {}".format(self))
+        self.master = master
+
+        self.columns = ("tag",)
+        self.tag_list = ttk.Treeview(
+            self,
+            show="headings",
+            height=10,
+            selectmode="browse",
+            columns=self.columns
+        )
+        self.tag_list.heading("tag", text="Tags")
+
+        # Add a scrollbar.
+        yscrollbar = ttk.Scrollbar(
+            self,
+            orient=tk.VERTICAL,
+            command=self.tag_list.yview
+        )
+        self.tag_list.configure(yscroll=yscrollbar.set)
+
+        # Layout the frame.
+        self.tag_list.grid(row=0, column=0, sticky="nswe")
+        yscrollbar.grid(row=0, column=1, sticky="ns")
+
+        self.refresh()
+
+    def refresh(self):
+        self.tag_list.delete(*self.tag_list.get_children())
+        for tag in self.master.dataset_model.list_tags():
+            self.tag_list.insert("", "end", values=[tag])
+
+
+class EditTagsFrame(ttk.Frame):
+    """Edit tags frame."""
+
+    def __init__(self, master, root, dataset_uri):
+        super().__init__(master)
+        logger.info("Initialising {}".format(self))
+        self.focus_set()
+        self.master = master
+        self.root = root
+
+        self.dataset_model = DataSetModel()
+        self.dataset_model.load_dataset(dataset_uri)
+
+        # Entry and button to add a tag.
+        vcmd = (self.master.register(self._validate_tag_callback), "%P")
+        self.tag_var = tk.StringVar()
+        self.tag_entry = ttk.Entry(
+            self,
+            width=10,
+            textvariable=self.tag_var,
+            validate="key",
+            validatecommand=vcmd
+        )
+        self.add_btn = ttk.Button(self, text="Add", command=self.put_tag)
+
+        # Treeview displaying the dataset's tags.
+        self.tag_list_frame = TagListFrame(self)
+
+        # Delete button.
+        self.delete_btn = ttk.Button(self, text="Delete", command=self.delete_tag)  # NOQA
+
+        # Layout the frame.
+        self.tag_entry.grid(row=0, column=0, sticky="nsew")
+        self.add_btn.grid(row=0, column=1, sticky="nsew")
+        self.tag_list_frame.grid(row=1, column=0, columnspan=2, sticky="nswe")
+        self.delete_btn.grid(row=2, column=0, columnspan=2, sticky="nsew")
+
+        self.refresh()
+
+    def _validate_tag_callback(self, name):
+        if (name is not None) and (len(name) > 0):
+            return dtoolcore.utils.name_is_valid(name)
+        return True
+
+    def put_tag(self):
+        logger.info("Putting tag: {}".format(self.tag_var.get()))
+        self.dataset_model.put_tag(self.tag_var.get())
+        self.refresh()
+        self.root.dataset_frame.refresh()
+
+    def delete_tag(self):
+        cur_tag = self.tag_list_frame.tag_list.focus()
+        values = self.tag_list_frame.tag_list.item(cur_tag, option="values")
+        if values == "":
+            return
+        tag = values[0]
+        logger.info("Deleting tag: {}".format(tag))
+        self.dataset_model.delete_tag(tag)
+        self.refresh()
+        self.root.dataset_frame.refresh()
+
+    def refresh(self):
+        self.tag_list_frame.refresh()
+
+
 class EditMetadataWindow(tk.Toplevel):
     """Edit metadata window."""
     def __init__(self, master, dataset_uri):
@@ -873,6 +1070,31 @@ class EditMetadataWindow(tk.Toplevel):
         self.destroy()
 
 
+class EditTagsWindow(tk.Toplevel):
+    """Edit tags window."""
+    def __init__(self, master, dataset_uri):
+        super().__init__(master)
+
+        self.root = master
+
+        # Stop user being able to interact with any other window.
+        self.grab_set()
+
+        # Implement custom behaviour when closing the window.
+        # Needed to set the App.edit_metadata_window to None.
+        self.protocol("WM_DELETE_WINDOW", self.dismiss)
+
+        ds_name = self.root.dataset_list_model.get_active_name()
+        self.title("Edit tags: {}".format(ds_name))
+        logger.info("Initialising {}".format(self))
+        edit_tags_frame = EditTagsFrame(self, master, dataset_uri)
+        edit_tags_frame.grid(row=0, column=0, sticky="nwes")
+
+    def dismiss(self):
+        self.root.edit_tags_window = None
+        self.destroy()
+
+
 class PreferencesWindow(tk.Toplevel):
     """Preferences window."""
 
@@ -906,19 +1128,23 @@ class PreferencesWindow(tk.Toplevel):
             text="Directory where datasets are stored."
         )
         self.lbl = ttk.Label(self.label_frame, text="Base URI")
-        self.entry = ttk.Entry(
-            self.label_frame,
-            width=len(self.root.base_uri_model.get_base_uri())
-        )
-        self.entry.insert(0, self.root.base_uri_model.get_base_uri())
-        self.entry.configure(state="readonly")
+
+        # Show the current base URI if set.
+        if self.root.base_uri_model.get_base_uri() is not None:
+            self.entry = ttk.Entry(
+                self.label_frame,
+                width=len(self.root.base_uri_model.get_base_uri())
+            )
+            self.entry.insert(0, self.root.base_uri_model.get_base_uri())
+            self.entry.configure(state="readonly")
+            self.entry.grid(row=1, column=0, sticky="ew")
+
         self.button = ttk.Button(
             self.label_frame,
             text="Select local base URI directory",
             command=self.select_local_base_uri_directory
         )
         self.help_lbl.grid(row=0, column=0, columnspan=2, sticky="ew")
-        self.entry.grid(row=1, column=0, sticky="ew")
         self.button.grid(row=1, column=1)
 
     def select_local_base_uri_directory(self):
@@ -973,6 +1199,7 @@ class App(tk.Tk):
         self.preferences_window = None
         self.edit_metadata_window = None
         self.active_dataset_metadata_supported = False
+        self.edit_tags_window = None
 
         # Make sure that the GUI expands/shrinks when the window is resized.
         self.columnconfigure(0, weight=1)
@@ -1038,6 +1265,14 @@ class App(tk.Tk):
             event_cmd=self._edit_metadata_event
         )
 
+        self._add_menu_command(
+            menu=menu_edit,
+            label="Edit tags...",
+            accelerator_key="T",
+            cmd=self.edit_tags,
+            event_cmd=self._edit_tags_event
+        )
+
         if self.platform != "aqua":
             self._add_menu_command(
                 menu=menu_edit,
@@ -1060,8 +1295,8 @@ class App(tk.Tk):
         self.mainframe.columnconfigure(2, weight=1)
         self.mainframe.rowconfigure(0, weight=1)
 
-        self.dataset_list_frame = DataSetListFrame(self.mainframe, self)
-        self.dataset_list_frame.grid(row=0, column=0, sticky="nsew")
+        self.dataset_collection_frame = DataSetCollectionFrame(self.mainframe, self)  # NOQA
+        self.dataset_collection_frame.grid(row=0, column=0, sticky="nsew")
 
         # Add vertical separator between dataset list and dataset views.
         ttk.Separator(self.mainframe, orient=tk.VERTICAL).grid(row=0, column=1, sticky="ns")  # NOQA
@@ -1129,12 +1364,27 @@ class App(tk.Tk):
         else:
             self.edit_metadata_window.focus_set()
 
+    def _edit_tags_event(self, tags):
+        self.edit_tags()
+
+    def edit_tags(self):
+        """Open window with form to edit a dataset's tags."""
+        logger.info(self.edit_tags.__doc__)
+        if self.edit_tags_window is None:
+            self.edit_tags_window = EditTagsWindow(
+                self,
+                self.dataset_list_model.get_active_uri()
+            )
+        else:
+            self.edit_tags_window.focus_set()
+
     def _quit_event(self, event):
         self.quit()
 
     def quit(self):
         """Quit the dtool-gui application."""
         logger.info(self.quit.__doc__)
+        self.destroy()
 
     def _show_perferences_window_event(self, event):
         self.show_perferences_window()
@@ -1159,7 +1409,7 @@ class App(tk.Tk):
     def refresh(self):
         """Refreshing all frames."""
         logger.info(self.refresh.__doc__)
-        self.dataset_list_frame.refresh()
+        self.dataset_collection_frame.refresh()
         self.dataset_frame.refresh()
 
 
